scaleecg<-function(data,smin,smax) {
	vmax<-max(data);
	vmin<-min(data);
	srng<-(smax-smin)/(vmax-vmin);
	return(smin+(data-vmin)*srng);
}
deriv2<-function(data) {
	d<-c();
	for(i in 1:(length(data)-4)) {
		d11<-data[i+1]-data[i];
		d12<-data[i+2]-data[i+1];
		d13<-data[i+3]-data[i+2];
		d14<-data[i+4]-data[i+3];
		d21<-d12-d11;
		d22<-d13-d12;
		d23<-d14-d13;
		d31<-d22-d21;
		d32<-d23-d22;
		d41<-d32-d31;
		d<-c(d,d21-d31+(d41*0.9166667));
	}
	return(d);
}
findr<-function(data,pc) {
	dqrs<-c();
	der<-filterecg(deriv2(data)^5,pc);
	p0<--1;
	for(i in 1:length(der)) {
		if (der[i] == -1) {
			if (p0<0) {
				p0<-i;
			}
			else {
				dqrs<-rbind(dqrs,c(p0,i));
				p0<-i;
			}
		}
	}
	dqrs<-as.data.frame(dqrs);
	names(dqrs)<-c("R0","R1");
	return(dqrs);
}
filterecg<-function(data,pc) {
	f<-c();
	sg<-0;
	for (i in 1:(length(data)-1)) {
		if (data[i] >= 0) {
			f<-c(f,0);
			sg<-0;
		}
		else {
			if (data[i] > pc) {
				f<-c(f,0);
				sg<-1;
			}
			else if (data[i+1]<=data[i]) {
				f<-c(f,0);
				sg<-1;
			}
			else if (sg == 1) {
				sg <-2;
				f<-c(f,-1);
			}
			else if (sg == 2) {
				f<- c(f,0);
			}
		} 
	}
	return(f);
}
mrqa<-function(data,clsval,edim=3,sdel=1,rad,emin=-1,emax=1,rr=0) {
	tmprad<- rad;
	repeat {			
		rqa<-crqa(data,
			data,
			delay=sdel,
			embed=edim,
			radius=tmprad,
			normalize=0,
			rescale=0,
			mindiagline=2,
			minvertline=2,
			side="lower");
		if (rr == 0) {
			break;
		}
		if (rqa$RR-rr < -0.05) {
			tmprad<-tmprad+0.0001;
		}
		else if (rqa$RR-rr > 0.05) {
			tmprad<-tmprad-0.0001;
		}
		else {
			break;
		}
	} 		
	vrqa<-c(rqa$RR,rqa$DET,rqa$NRLINE,rqa$maxL,
		rqa$L,rqa$ENTR,rqa$rENTR,rqa$LAM,rqa$TT,clsval);
	return(vrqa);
}
rqadata<-function(vrqa,signalf,intf,clsval,edim=3,sdel=1,rad,emin=-1,emax=1,rr=0) {
	dint<-read.csv2(intf);
	dsig<-read.csv2(signalf);
	dnorm<-scaleecg(dsig[,1],emin,emax);
	for(j in 1:length(dint[,1])) {		
		rqa<-mrqa(dnorm[dint[j,1]:dint[j,2]],clsval,edim,sdel,rad,emin,emax,rr);
		if (is.null(vrqa)) {
			vrqa<-rqa;
		}
		else {
			vrqa<-rbind(vrqa,rqa);
		}
	}
	return(vrqa);
}
rqarnd<-function(vrqa,signalf,intf,edim=3,sdel=1,rad,emin=-1,emax=1,rr=0) {
	dint<-read.csv2(intf);
	dsig<-read.csv2(signalf);
	dnorm<-scaleecg(dsig[,1],emin,emax);
	s1<-1:length(dint[,1]);
	s2<-sample(s1,length(s1)/2);
	s1<-s1[-s2];
	for(j in s1) {
		rqa<-mrqa(dnorm[dint[j,1]:dint[j,2]],1,edim,sdel,rad,emin,emax,rr);
		if (is.null(vrqa)) {
			vrqa<-rqa;
		}
		else {
			vrqa<-rbind(vrqa,rqa);
		}
	}
	for(j in s2) {
		rqa<-mrqa(dnorm[dint[j,1]:dint[j,2]],2,edim,sdel,rad,emin,emax,rr);
		if (is.null(vrqa)) {
			vrqa<-rqa;
		}
		else {
			vrqa<-rbind(vrqa,rqa);
		}
	}
	return(vrqa);
}
rqaset<-function() {
	data<-rqadata(NULL,"Subject27_2.csv","Subject27_2.int",clsval=1,edim=2,sdel=15,rad=0.01,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject03_2.csv","Subject03_2.int",clsval=1,edim=2,sdel=15,rad=0.03,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject17_2.csv","Subject17_2.int",clsval=1,edim=2,sdel=15,rad=0.03,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject12_2.csv","Subject12_2.int",clsval=1,edim=2,sdel=15,rad=0.019,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject10_2.csv","Subject10_2.int",clsval=2,edim=2,sdel=15,rad=0.0125,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject21_2.csv","Subject21_2.int",clsval=2,edim=2,sdel=15,rad=0.025,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject22_2.csv","Subject22_2.int",clsval=2,edim=2,sdel=15,rad=0.0181,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject09_2.csv","Subject09_2.int",clsval=2,edim=2,sdel=15,rad=0.0122,emin=-1,emax=1,rr=1.25);
	return(data);
}
rqaset2<-function() {
	data<-rqadata(NULL,"Subject27_2.csv","Subject27_2.int",clsval=1,edim=2,sdel=15,rad=0.01,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject03_2.csv","Subject03_2.int",clsval=2,edim=2,sdel=15,rad=0.03,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject17_2.csv","Subject17_2.int",clsval=1,edim=2,sdel=15,rad=0.03,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject12_2.csv","Subject12_2.int",clsval=2,edim=2,sdel=15,rad=0.019,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject10_2.csv","Subject10_2.int",clsval=1,edim=2,sdel=15,rad=0.0125,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject21_2.csv","Subject21_2.int",clsval=2,edim=2,sdel=15,rad=0.025,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject22_2.csv","Subject22_2.int",clsval=1,edim=2,sdel=15,rad=0.0181,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject09_2.csv","Subject09_2.int",clsval=2,edim=2,sdel=15,rad=0.0122,emin=-1,emax=1,rr=1.25);
	return(data);
}
rqasettest<-function() {
	data<-rqadata(NULL,"Subject15_2.csv","Subject15_2.int",clsval=1,edim=2,sdel=15,rad=0.0106,emin=-1,emax=1,rr=1.25);
	data<-rqadata(data,"Subject04_2.csv","Subject04_2.int",clsval=2,edim=2,sdel=15,rad=0.0262,emin=-1,emax=1,rr=1.25);
	return(data);
}
plotRec<-function(sig,rp) {
	plot(sig,type="o");
	par(new=T);
	y<-c();
	for(ix in 1:nrow(rp)) {
		for(r in 1:nrow(rp)) {
			if (rp[r,ix]>0) {
				y<-c(y,r,(nrow(rp)+1)-ix);
			}
		}
	}
	points(y,sig[y],col="red");
}
